package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.service.product.proxy.WareServiceProxy;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.WareDTO;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.data.api.product.vo.WareVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@RestController
@RequestMapping(value = "/ware")
public class WareController {

    @Autowired
    private WareServiceProxy wareService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<WareVo>> search(SearchDTO searchDTO) {
        return wareService.search(searchDTO);
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<WareVo> create(@Valid @RequestBody WareDTO wareDTO) {
        return wareService.create(wareDTO);
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<WareVo> update(@Valid @RequestBody WareDTO wareDTO) {
        if (wareDTO.getId() == null) {
            return APIResult.error(APICode.error("参数错误: id"));
        }
        return wareService.update(wareDTO);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("id") String id) {
        return wareService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<WareVo> getWare(@PathVariable("id") String id) {
        return wareService.getWare(id);
    }
}
