package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.WareDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.Courseware;
import cn.rankin.productservice.repository.WareRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class CoursewareService {

    @Autowired
    private WareRepository wareRepository;

    public APIResult<Page<Courseware>> search(Courseware Courseware, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = wareRepository.count(Courseware);
        Page<Courseware> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Courseware> coursewareList = wareRepository.find(Courseware, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(coursewareList);

        return APIResult.ok(page);
    }

    public boolean exists(String code) {
        Long count = wareRepository.countByCode(code);
        return count > 0L;
    }

    @Transactional
    public APIResult<Courseware> create(WareDTO wareDTO) {
        Courseware Courseware = convert(wareDTO);
        if (exists(Courseware.getCode())) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }
        Courseware result = wareRepository.save(Courseware);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Courseware> update(WareDTO wareDTO) {
        Courseware Courseware = convert(wareDTO);
        Courseware result = wareRepository.update(Courseware);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = wareRepository.deleteAllById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<Courseware> findById(String id) {
        Courseware Courseware = wareRepository.find(id);
        if (Courseware == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(Courseware);
    }

    public APIResult<List<Courseware>> findByIds(List<String> ids) {
        List<Courseware> coursewareList = wareRepository.findByIdIn(ids, BaseStatusEnum.NORMAL);

        if (coursewareList == null || coursewareList.size() == 0) {
            return APIResult.ok(coursewareList);
        }

        Map<String, Courseware> wareMap = ListUtil.convert(coursewareList, "id", Courseware.class);
        List<Courseware> sortCoursewareList = new ArrayList<>();
        ids.forEach( id -> {
            Courseware Courseware = wareMap.get(id);
            if (Courseware != null) {
                sortCoursewareList.add(Courseware);
            }
        });

        return APIResult.ok(sortCoursewareList);
    }
}
