package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.PackageProductDTO;
import cn.rankin.data.api.product.entity.PackageProductRelation;
import cn.rankin.productservice.repository.PackageProductRelationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class PackageProductRelationService {

    @Autowired
    private PackageProductRelationRepository packageProductRelationRepository;

    public List<PackageProductRelation> findByPackageId(String packageId) {
        return packageProductRelationRepository.findByPackageId(packageId);
    }

    @Transactional
    public Integer deleteByPackageId(String pkgId) {
        return packageProductRelationRepository.deleteByPackageId(pkgId);
    }

    @Transactional
    public void updateRelation(String pkgId, List<PackageProductDTO> packageProductDTOList) {
        // 记录顺序数组: 最新的
        List<String> productIdList = new ArrayList<>();
        // 记录最新的定价map
        Map<String, PackageProductDTO> packageProductDTOMap = new HashMap<>();

        packageProductDTOList.forEach( packageProductDTO -> {
            String productId = packageProductDTO.getPid();
            productIdList.add(productId);
            packageProductDTOMap.put(productId, packageProductDTO);
        });

        // 更新现有的, 同时记录现有的关系
        List<String> currentIdList = new ArrayList<>();
        List<PackageProductRelation> currentRelationList = packageProductRelationRepository.findByPackageId(pkgId);
        currentRelationList.forEach( relation -> {
            String productId = relation.getPid();
            if (!productIdList.contains(productId)) {
                relation.setStatus(BaseStatusEnum.DEL);
            }else {
                relation.setStatus(BaseStatusEnum.NORMAL);
                PackageProductDTO packageProductDTO = packageProductDTOMap.get(productId);
                relation.setCpPrice(packageProductDTO.getCpPrice());
            }
            currentIdList.add(productId);
        });

        // 新加的
        List<String> newAddProductIdList = ListUtil.subtract(productIdList, currentIdList);
        if (newAddProductIdList != null && newAddProductIdList.size() > 0) {
            newAddProductIdList.forEach( productId -> {
                PackageProductDTO packageProductDTO = packageProductDTOMap.get(productId);
                currentRelationList.add(convert(pkgId, packageProductDTO));
            });
        }

        // 排序
        currentRelationList.forEach( relation -> {
            String productId = relation.getPid();
            Integer sort = productIdList.indexOf(productId);
            relation.setSort(sort);
        });

        packageProductRelationRepository.save(currentRelationList);

    }
}
