package cn.rankin.tradeservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.OrderStatusEnum;
import cn.rankin.data.api.trade.dto.OrderSearchDTO;
import cn.rankin.data.api.trade.dto.UserOrderDTO;
import cn.rankin.data.api.trade.entity.UserOrder;
import cn.rankin.tradeservice.service.OrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Date;

@RestController
@RequestMapping(value = "/order")
public class OrderController {

    @InitBinder
    public void initBinder(WebDataBinder binder) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        dateFormat.setLenient(false);
        binder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));   //true:允许输入空值，false:不能为空值
    }

    @Autowired
    private OrderService orderService;

    // cms web需要做前置校验
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<UserOrder> create(@RequestBody UserOrderDTO userOrderDTO) {
        return orderService.create(userOrderDTO);
    }

    @RequestMapping(value = "/{orderId}", method = RequestMethod.GET)
    public APIResult<UserOrder> getOrder(@PathVariable String orderId) {
        return orderService.getOrder(orderId);
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<UserOrder>> search(OrderSearchDTO searchDTO) {
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/pay/{orderId}", method = RequestMethod.POST)
    public APIResult<Boolean> pay(@PathVariable("orderId") String orderId, @RequestParam("status") OrderStatusEnum status) {
        return orderService.pay(orderId, status);
    }

    @RequestMapping(value = "/pay/{orderId}", method = RequestMethod.DELETE)
    public APIResult<Boolean> cancel(@PathVariable("orderId") String orderId) {
        return orderService.cancel(orderId);
    }
}
