package cn.rankin.userservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.user.entity.Merchant;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;

@Repository
public interface MerchantRepository  extends BasicJpaRepository<Merchant, String> {

    Long countByCode(String code);

	List<Merchant> findByIdIn(List<String> ids);

	Merchant findByIdAndStatus(String id, BaseStatusEnum status);

	Merchant findByName(String name);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query("update Merchant m set m.status = :status where m.id in (:ids) ")
    int updateStatusByIds(@Param("ids") List<String> ids, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query("update Merchant m set m.status = :status where m.id = :id ")
    int updateStatusById(@Param("id") String id, @Param("status") BaseStatusEnum status);

    @Modifying(clearAutomatically = true)
    @Query(value = "update Merchant m set m.balance = m.balance - ?2 where m.balance - ?2 > 0 and m.id = ?1")
    Integer pay(String merchantId, BigDecimal quantity);
}
